//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef SYMMETRIC_TENSOR_HPP
#define SYMMETRIC_TENSOR_HPP

#include <QTextStream>
#include <Vector.hpp>

namespace mgx {
namespace util {
typedef Vector<3, float> Point3f;

class SymmetricTensor {
public:
  SymmetricTensor()
    : _ev1(1, 0, 0)
    , _ev2(0, 1, 0)
    , _evals(1, 1, 1)
  {
  }

  SymmetricTensor(const Point3f& ev1, const Point3f& ev2, const Point3f& evals)
    : _ev1(ev1)
    , _ev2(ev2)
    , _evals(evals)
  {
  }

  SymmetricTensor(const SymmetricTensor& copy)
    : _ev1(copy._ev1)
    , _ev2(copy._ev2)
    , _evals(copy._evals)
  {
  }

  SymmetricTensor& operator=(const SymmetricTensor& other)
  {
    _ev1 = other._ev1;
    _ev2 = other._ev2;
    _evals = other._evals;
    return *this;
  }

  Point3f& ev1() {
    return _ev1;
  }
  Point3f& ev2() {
    return _ev2;
  }
  const Point3f& ev1() const {
    return _ev1;
  }
  const Point3f& ev2() const {
    return _ev2;
  }
  Point3f ev3() const {
    return _ev1 ^ _ev2;
  }
  const Point3f& evals() const {
    return _evals;
  }
  Point3f& evals() {
    return _evals;
  }

protected:
  Point3f _ev1, _ev2;
  Point3f _evals;
};
} // namespace util
} // namespace mgx
#endif
