#ifndef THRUST_HPP
#define THRUST_HPP

#include "Config.hpp"
#include <thrust/version.h>

#define USE_OPENMP

#if THRUST_VERSION >= 100600
#    if defined(USE_TBB)
#        define USING_TBB
#        include <thrust/system/tbb/vector.h>
#        include <thrust/iterator/retag.h>
#        define THRUST_RETAG(x) thrust::reinterpret_tag<thrust::tbb::tag>(x)
#        define THRUST_TAG "Threading Building Blocks"
#    elif defined(USE_OPENMP)
#        define USING_OPENMP
#        include <thrust/system/omp/vector.h>
#        include <thrust/iterator/retag.h>
#        define THRUST_RETAG(x) thrust::reinterpret_tag<thrust::omp::tag>(x)
#        define THRUST_TAG "OpenMP"
#    else
#        include <thrust/system/cpp/vector.h>
#        include <thrust/iterator/retag.h>
#        define THRUST_RETAG(x) thrust::reinterpret_tag<thrust::cpp::tag>(x)
#        define THRUST_TAG "Plain C++"
#    endif
#else
#    define THRUST_RETAG(x) x
#    define THRUST_TAG "Plain C++"
#endif

#endif // THRUST_HPP

