//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef VERTEXDATA_HPP
#define VERTEXDATA_HPP

/**
 * \file VertexData.hpp
 *
 * This file contains the definition of a vertex.
 */

#include <Config.hpp>

#include <Geometry.hpp>

namespace mgx {
/**
 * \class VertexData VertexData.hpp <VertexData.hpp>
 *
 * This class defines the content of a vertex. That is all properties available
 * directly from a vertex.
 */
class VertexData {
public:
  Point3f pos;     ///< Position in space
  Point3f nrml;    ///< Normal vector
  Point2f txpos;   ///< Texture coordinates
  float tsignal;   ///< Temp signal
  float signal;    ///< Projected signal
  int label;       ///< Region to which vertex belongs
  char type;       ///< Type of vertex
  float area;      ///< Area of cell
  int saveId;      ///< Used when saving vertex
  int labcount;    ///< # of labels (cells) vertex connected to
  bool margin;     ///< Is vertex on the margin?
  bool inqueue;    ///< Is this in the queue? Used for segmentation
  bool selected;   ///< Is vertex selected?
  ulong minb;      ///< Closest border vertex

  // Constructor, set initial values
  VertexData()
    : pos(0, 0, 0)
    , nrml(0, 0, 0)
    , txpos(0, 0)
    , tsignal(0)
    , signal(1)
    , label(0)
    , type('j')
    , area(0)
    , saveId(0)
    , labcount(0)
    , margin(false)
    , inqueue(false)
    , selected(false)
    , minb(0)
  {
  }

  virtual ~VertexData() {
  }
};
} // namespace mgx

#endif /*VERTEX_DATA_H*/
