//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef CU_EXPORT_H
#define CU_EXPORT_H

#include <Config.hpp>
#include <cuda/CudaGlobal.hpp>
#include <thrust/host_vector.h>

#include <Matrix.hpp>
#include <Vector.hpp>
// Typedefs for host pinned memory
#include <thrust/system/cuda/experimental/pinned_allocator.h>

namespace mgx {
  typedef unsigned char ubyte;
  typedef unsigned int uint;
  typedef unsigned short ushort;
  typedef unsigned long ulong;
  typedef util::Vector<3, float> Point3f;
  typedef util::Vector<4, float> Point4f;
  typedef util::Vector<6, float> Point6f;
  typedef util::Vector<12, float> Point12f;
  // typedef Matrix<4, 4, float> Matrix4f;
  typedef util::Vector<3, int> Point3i;
  typedef util::Vector<3, uint> Point3u;
  typedef util::Vector<3, ulong> Point3ul;
  typedef util::Vector<3, ushort> Point3us;
  typedef thrust::host_vector<ubyte> HVecUB;
  typedef thrust::host_vector<uint> HVecU;
  typedef thrust::host_vector<ushort> HVecUS;
  typedef thrust::host_vector<float> HVecF;
  typedef thrust::host_vector<Point3u> HVec3U;
  typedef thrust::host_vector<Point3f> HVec3F;
  typedef thrust::host_vector<Point4f> HVec4F;
  
  typedef thrust::host_vector<float, thrust::cuda::experimental::pinned_allocator<float> > HPVecF;
  typedef thrust::host_vector<Point6f, thrust::cuda::experimental::pinned_allocator<Point6f> > HPVec6F;
  typedef thrust::host_vector<ushort, thrust::cuda::experimental::pinned_allocator<ushort> > HPVecUS;
} // namespace mgx

cuda_EXPORT int initGPU();
cuda_EXPORT size_t memAvailGPU();
cuda_EXPORT void freeMemGPU();
cuda_EXPORT void holdMemGPU();
cuda_EXPORT int setHoldMemGPU(uint holdmem);

cuda_EXPORT int averageGPU(const mgx::Point3i& imgSize, const mgx::Point3i& radius,
  const mgx::HVecUS& srcdata, mgx::HVecUS& dstdata);
cuda_EXPORT int dilateGPU(const mgx::Point3i& imgSize, const mgx::Point3i& radius, 
  const mgx::HVecUS& srcdata, mgx::HVecUS& dstdata);
cuda_EXPORT int erodeGPU(const mgx::Point3i& imgSize, const mgx::Point3i& radius, 
  const mgx::HVecUS& srcdata, mgx::HVecUS& dstdata, bool byLabel);
cuda_EXPORT int gaussianBlurGPU(const mgx::Point3i& imgSize, const mgx::Point3f& imgStep,
  const mgx::Point3f& sigma, const mgx::HVecUS& srcdata, mgx::HVecUS& dstdata);
cuda_EXPORT int sharpenGPU(const mgx::Point3i& imgSize, const mgx::Point3f& imgStep,
  const mgx::Point3f& sigma, const float amount, const mgx::HVecUS& srcdata, mgx::HVecUS& dstdata);
cuda_EXPORT int applyKernelGPU(const mgx::Point3i& imgSize, const mgx::HVecF& kernelX,
  const mgx::HVecF& kernelY, const mgx::HVecF& kernelZ,
  const mgx::HVecUS& srcdata, mgx::HVecUS& dstdata);
cuda_EXPORT int colorGradGPU(const mgx::Point3i& imgSize, float div, const mgx::HVecUS& srcdata,
  mgx::HVecUS& dstdata);
cuda_EXPORT int edgeDetectGPU(const mgx::Point3i& imgSize, ushort lowthresh, ushort highthresh, 
  float mult, ushort fillval, const mgx::HVecUS& srcdata, mgx::HVecUS& dstdata);
cuda_EXPORT int insideMeshGPU(const mgx::Point3i& base, const mgx::Point3i& size, 
  const mgx::Point3f& step, const mgx::Point3f& shift, const mgx::HVec3F& pts, 
  const mgx::HVec3U& tris, const mgx::HVec3F& nrmls, mgx::HVecUS& dstdata);
cuda_EXPORT int nearMeshGPU(const mgx::Point3i& base, const mgx::Point3i& size, 
  const mgx::Point3f& step, const mgx::Point3f& shift, const mgx::HVec3F& pts, 
  const mgx::HVec3F& nrmls, float minDist, float maxDist, mgx::HVecUS& dstdata);
cuda_EXPORT int clipStackGPU(const mgx::Point3i& imgSize, const mgx::Point3f& step, 
  const mgx::Point3f& shift, const mgx::Point3u& clipDo, mgx::HVec4F& pn, 
  const mgx::HVecUS& srcdata, mgx::HVecUS& dstdata);
cuda_EXPORT int pixelEditGPU(const mgx::Point3i& base, const mgx::Point3i& size, 
  const mgx::Point3i& imgSize, const mgx::Point3f& step, const mgx::Point3f& shift, 
  const mgx::Point3f& p, const mgx::Point3f& pz, float radius, ushort color, 
  const mgx::Point3u& clipDo, const mgx::HVec4F& pn, mgx::HVecUS& data);
#endif
