//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef ITKSINK_HPP
#define ITKSINK_HPP

#include <ITKConfig.hpp>
#include <ITKProcess.hpp>

namespace mgx {
namespace process {
class mgxITK_EXPORT ITKVTKWriter : public StackProcess {
public:
  ITKVTKWriter(const StackProcess& process)
    : Process(process)
    , StackProcess(process)
  {
  }

  bool operator()(const QStringList& parms)
  {
    if(!checkState().store(STORE_NON_EMPTY))
      return false;
    Stack* s = currentStack();
    Store* input = s->currentStore();
    bool res = (*this)(input, parms[0]);
    return res;
  }

  bool operator()(const Store* input, QString filename);

  QString name() const {
    return "ITK VTK Writer";
  }
  QString folder() const {
    return "ITK/System";
  }
  QString description() const {
    return "Write a VTK Image File";
  }
  QStringList parmNames() const {
    return QStringList() << "Filename";
  }
  QStringList parmDefaults() const {
    return QStringList() << "";
  }
  QIcon icon() const {
    return QIcon(":/images/save.png");
  }
};
}
}

#endif // ITKSINK_HPP
