//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef STACKPROCESSANALYSES_HPP
#define STACKPROCESSANALYSES_HPP

#include <Process.hpp>

namespace mgx {
namespace process {

///\addtogroup StackProcess
///@{
/**
 * \class ComputeVolume StackProcessAnalyses.hpp <StackProcessAnalyses.hpp>
 *
 * Compute the volume of each label and write the result in a CSV file.
 *
 * The volume is computed as the product between the number of voxel for each
 * label, and the volume of a single voxel.
 */
class mgxBase_EXPORT ComputeVolume : public StackProcess {
public:
  ComputeVolume(const StackProcess& process)
    : Process(process)
    , StackProcess(process)
  {
  }

  bool operator()(const QStringList& parms)
  {
    if(!checkState().store(STORE_NON_EMPTY | STORE_LABEL))
      return false;
    Store* input = currentStack()->currentStore();
    return (*this)(input, parms[0]);
  }

  bool operator()(const Store* input, QString filename);

  QString name() const {
    return "Compute Volumes";
  }
  QString description() const
  {
    return "Compute the volumes of the labels, i.e. the number of voxels multiplied by the vocel size.";
  }
  QString folder() const {
    return "Analyses";
  }
  QStringList parmNames() const {
    return QStringList() << "Filename";
  }
  QStringList parmDescs() const {
    return QStringList() << "Filename";
  }
  QStringList parmDefaults() const {
    return QStringList() << "output.csv";
  }
  QIcon icon() const {
    return QIcon(":/images/CellFiles3D.png");
  }
};
///@}
} // namespace process
} // namespace mgx

#endif
